@@ -688,28 +688,28 @@ divll_pre(ulong a_lo, ulong n, ulong ninv)
 
 #endif
 #ifdef LONG_IS_64BIT
-#define __AGM_ATAN_LIMIT                 60
-#define __DIVRR_GMP_LIMIT                4
-#define __EXPNEWTON_LIMIT                66
+#define __AGM_ATAN_LIMIT                 3840
+#define __DIVRR_GMP_LIMIT                256
+#define __EXPNEWTON_LIMIT                4224
 #define __EXTGCD_HALFGCD_LIMIT           -1
 #define __F2x_MUL_KARATSUBA_LIMIT        11
 #define __F2x_MUL_MULII_LIMIT            41
 #define __F2xqX_BARRETT_LIMIT            48
 #define __F2xqX_DIVREM_BARRETT_LIMIT     97
-#define __F2xqX_EXTGCD_LIMIT             97
-#define __F2xqX_GCD_LIMIT                605
-#define __F2xqX_HALFGCD_LIMIT            127
+#define __F2xqX_EXTGCD_LIMIT             10
+#define __F2xqX_GCD_LIMIT                310
+#define __F2xqX_HALFGCD_LIMIT            89
 #define __F2xqX_INVBARRETT_LIMIT         50
 #define __F2xqX_REM_BARRETT_LIMIT        101
 #define __Flx_BARRETT2_LIMIT             31
 #define __Flx_BARRETT_LIMIT              92
 #define __Flx_DIVREM2_BARRETT_LIMIT      14
 #define __Flx_DIVREM_BARRETT_LIMIT       161
-#define __Flx_EXTGCD2_LIMIT              61
-#define __Flx_EXTGCD_LIMIT               158
-#define __Flx_GCD2_LIMIT                 1409
-#define __Flx_GCD_LIMIT                  852
-#define __Flx_HALFGCD2_LIMIT             36
+#define __Flx_EXTGCD2_LIMIT              22
+#define __Flx_EXTGCD_LIMIT               17
+#define __Flx_GCD2_LIMIT                 937
+#define __Flx_GCD_LIMIT                  426
+#define __Flx_HALFGCD2_LIMIT             31
 #define __Flx_HALFGCD_LIMIT              120
 #define __Flx_INVBARRETT2_LIMIT          22
 #define __Flx_INVBARRETT_LIMIT           200
@@ -725,64 +725,65 @@ divll_pre(ulong a_lo, ulong n, ulong ninv)
 #define __Flx_SQR_SQRI_LIMIT             37
 #define __FlxqX_BARRETT_LIMIT            17
 #define __FlxqX_DIVREM_BARRETT_LIMIT     46
-#define __FlxqX_EXTGCD_LIMIT             44
-#define __FlxqX_GCD_LIMIT                470
-#define __FlxqX_HALFGCD_LIMIT            60
+#define __FlxqX_EXTGCD_LIMIT             21
+#define __FlxqX_GCD_LIMIT                116
+#define __FlxqX_HALFGCD_LIMIT            30
 #define __FlxqX_INVBARRETT_LIMIT         22
 #define __FlxqX_REM_BARRETT_LIMIT        48
 #define __FpXQX_BARRETT_LIMIT            12
 #define __FpXQX_DIVREM_BARRETT_LIMIT     30
-#define __FpXQX_EXTGCD_LIMIT             28
-#define __FpXQX_GCD_LIMIT                191
-#define __FpXQX_HALFGCD_LIMIT            35
+#define __FpXQX_EXTGCD_LIMIT             10
+#define __FpXQX_GCD_LIMIT                101
+#define __FpXQX_HALFGCD_LIMIT            29
 #define __FpXQX_INVBARRETT_LIMIT         40
 #define __FpXQX_REM_BARRETT_LIMIT        30
 #define __FpX_BARRETT_LIMIT              38
 #define __FpX_DIVREM_BARRETT_LIMIT       113
-#define __FpX_EXTGCD_LIMIT               87
-#define __FpX_GCD_LIMIT                  406
-#define __FpX_HALFGCD_LIMIT              58
+#define __FpX_EXTGCD_LIMIT               19
+#define __FpX_GCD_LIMIT                  194
+#define __FpX_HALFGCD_LIMIT              45
 #define __FpX_INVBARRETT_LIMIT           111
 #define __FpX_REM_BARRETT_LIMIT          111
 #define __Fp_POW_BARRETT_LIMIT           127
 #define __Fp_POW_REDC_LIMIT              17
 #define __GCD_HALFGCD_LIMIT              -1
-#define __HALFGCD_LIMIT                  12
+#define __HALFGCD_LIMIT                  66
 #define __INVMOD_GMP_LIMIT               3
-#define __INVNEWTON_LIMIT                75
-#define __LOGAGMCX_LIMIT                 22
-#define __LOGAGM_LIMIT                   6
+#define __INVNEWTON_LIMIT                4800
+#define __LOGAGMCX_LIMIT                 1408
+#define __LOGAGM_LIMIT                   384
 #define __MULII_FFT_LIMIT                -1
 #define __MULII_KARATSUBA_LIMIT          -1
-#define __MULRR_MULII_LIMIT              55
+#define __MULRR_MULII_LIMIT              3520
 #define __RgX_MUL_LIMIT                  9
 #define __RgX_SQR_LIMIT                  38
 #define __SQRI_FFT_LIMIT                 -1
 #define __SQRI_KARATSUBA_LIMIT           -1
-#define __SQRR_SQRI_LIMIT                12
+#define __SQRR_SQRI_LIMIT                768
+#define __ZM2_MUL_LIMIT                  14
 #else
-#define __AGM_ATAN_LIMIT                 89
-#define __DIVRR_GMP_LIMIT                4
-#define __EXPNEWTON_LIMIT                197
+#define __AGM_ATAN_LIMIT                 2848
+#define __DIVRR_GMP_LIMIT                128
+#define __EXPNEWTON_LIMIT                6304
 #define __EXTGCD_HALFGCD_LIMIT           -1
 #define __F2x_MUL_KARATSUBA_LIMIT        13
 #define __F2x_MUL_MULII_LIMIT            774
 #define __F2xqX_BARRETT_LIMIT            48
 #define __F2xqX_DIVREM_BARRETT_LIMIT     127
-#define __F2xqX_EXTGCD_LIMIT             127
-#define __F2xqX_GCD_LIMIT                884
-#define __F2xqX_HALFGCD_LIMIT            89
+#define __F2xqX_EXTGCD_LIMIT             10
+#define __F2xqX_GCD_LIMIT                257
+#define __F2xqX_HALFGCD_LIMIT            61
 #define __F2xqX_INVBARRETT_LIMIT         40
 #define __F2xqX_REM_BARRETT_LIMIT        127
 #define __Flx_BARRETT2_LIMIT             52
 #define __Flx_BARRETT_LIMIT              164
 #define __Flx_DIVREM2_BARRETT_LIMIT      111
 #define __Flx_DIVREM_BARRETT_LIMIT       470
-#define __Flx_EXTGCD2_LIMIT              184
-#define __Flx_EXTGCD_LIMIT               469
-#define __Flx_GCD2_LIMIT                 1281
-#define __Flx_GCD_LIMIT                  2817
-#define __Flx_HALFGCD2_LIMIT             181
+#define __Flx_EXTGCD2_LIMIT              24
+#define __Flx_EXTGCD_LIMIT               21
+#define __Flx_GCD2_LIMIT                 621
+#define __Flx_GCD_LIMIT                  1246
+#define __Flx_HALFGCD2_LIMIT             119
 #define __Flx_HALFGCD_LIMIT              586
 #define __Flx_INVBARRETT2_LIMIT          397
 #define __Flx_INVBARRETT_LIMIT           501
@@ -798,41 +799,42 @@ divll_pre(ulong a_lo, ulong n, ulong ninv)
 #define __Flx_SQR_SQRI_LIMIT             183
 #define __FlxqX_BARRETT_LIMIT            17
 #define __FlxqX_DIVREM_BARRETT_LIMIT     46
-#define __FlxqX_EXTGCD_LIMIT             44
-#define __FlxqX_GCD_LIMIT                1289
-#define __FlxqX_HALFGCD_LIMIT            89
+#define __FlxqX_EXTGCD_LIMIT             19
+#define __FlxqX_GCD_LIMIT                245
+#define __FlxqX_HALFGCD_LIMIT            53
 #define __FlxqX_INVBARRETT_LIMIT         22
 #define __FlxqX_REM_BARRETT_LIMIT        48
 #define __FpXQX_BARRETT_LIMIT            12
 #define __FpXQX_DIVREM_BARRETT_LIMIT     30
-#define __FpXQX_EXTGCD_LIMIT             28
-#define __FpXQX_GCD_LIMIT                182
-#define __FpXQX_HALFGCD_LIMIT            35
+#define __FpXQX_EXTGCD_LIMIT             10
+#define __FpXQX_GCD_LIMIT                116
+#define __FpXQX_HALFGCD_LIMIT            30
 #define __FpXQX_INVBARRETT_LIMIT         40
 #define __FpXQX_REM_BARRETT_LIMIT        30
 #define __FpX_BARRETT_LIMIT              44
 #define __FpX_DIVREM_BARRETT_LIMIT       116
-#define __FpX_EXTGCD_LIMIT               81
-#define __FpX_GCD_LIMIT                  414
-#define __FpX_HALFGCD_LIMIT              55
+#define __FpX_EXTGCD_LIMIT               10
+#define __FpX_GCD_LIMIT                  234
+#define __FpX_HALFGCD_LIMIT              53
 #define __FpX_INVBARRETT_LIMIT           121
 #define __FpX_REM_BARRETT_LIMIT          127
 #define __Fp_POW_BARRETT_LIMIT           11
 #define __Fp_POW_REDC_LIMIT              3
 #define __GCD_HALFGCD_LIMIT              -1
-#define __HALFGCD_LIMIT                  22
+#define __HALFGCD_LIMIT                  58
 #define __INVMOD_GMP_LIMIT               3
-#define __INVNEWTON_LIMIT                93
-#define __LOGAGMCX_LIMIT                 32
-#define __LOGAGM_LIMIT                   45
+#define __INVNEWTON_LIMIT                2976
+#define __LOGAGMCX_LIMIT                 1024
+#define __LOGAGM_LIMIT                   1440
 #define __MULII_FFT_LIMIT                -1
 #define __MULII_KARATSUBA_LIMIT          -1
-#define __MULRR_MULII_LIMIT              19
+#define __MULRR_MULII_LIMIT              608
 #define __RgX_MUL_LIMIT                  7
 #define __RgX_SQR_LIMIT                  34
 #define __SQRI_FFT_LIMIT                 -1
 #define __SQRI_KARATSUBA_LIMIT           -1
-#define __SQRR_SQRI_LIMIT                9
+#define __SQRR_SQRI_LIMIT                288
+#define __ZM2_MUL_LIMIT                  15
 #endif
 #line 2 "../src/kernel/gmp/int.h"
 /* Copyright (C) 2000  The PARI group.
@@ -891,6 +893,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
  * These functions can be inline; they are also defined externally in
  * mpinl.c, which includes this file and never needs to be changed */
 
+INLINE long
+nbits2lg(long x) {
+  return (long)(((ulong)x+3*BITS_IN_LONG-1) >> TWOPOTBITS_IN_LONG);
+}
+INLINE long
+lg2prec(long x){ return (x-2) * BITS_IN_LONG; }
+
 INLINE long
 evallg(long x)
 {
@@ -905,6 +914,13 @@ evalvalp(long x)
   return v;
 }
 INLINE long
+evalvalser(long x)
+{
+  long v = _evalvalser(x);
+  if (v & ~VALSERBITS) pari_err_OVERFLOW("valser()");
+  return v;
+}
+INLINE long
 evalexpo(long x)
 {
   long v = _evalexpo(x);
@@ -992,6 +1008,12 @@ INLINE GEN
 gc_NULL(pari_sp av) { set_avma(av); return NULL; }
 INLINE GEN
 gc_const(pari_sp av, GEN x) { set_avma(av); return x; }
+INLINE GEN
+gc_stoi(pari_sp av, long x) { set_avma(av); return stoi(x); }
+INLINE GEN
+gc_utoi(pari_sp av, ulong x) { set_avma(av); return utoi(x); }
+INLINE GEN
+gc_utoipos(pari_sp av, ulong x) { set_avma(av); return utoipos(x); }
 
 INLINE GEN
 new_chunk(size_t x) /* x is a number of longs */
@@ -1092,15 +1114,17 @@ cgetineg(long x)
 INLINE GEN
 cgetr_block(long x)
 {
-  GEN z = newblock((size_t)x);
-  z[0] = CLONEBIT | evaltyp(t_REAL) | evallg(x);
+  long l = nbits2lg(x);
+  GEN z = newblock((size_t)l);
+  z[0] = CLONEBIT | evaltyp(t_REAL) | evallg(l);
   return z;
 }
 INLINE GEN
 cgetr(long x)
 {
-  GEN z = new_chunk((size_t)x);
-  z[0] = evaltyp(t_REAL) | evallg(x);
+  long l = nbits2lg(x);
+  GEN z = new_chunk((size_t)l);
+  z[0] = evaltyp(t_REAL) | evallg(l);
   return z;
 }
 
@@ -1281,25 +1305,25 @@ umuluu_le(ulong x, ulong y, ulong n)
 }
 
 INLINE GEN
-real_0_bit(long bitprec) { GEN x=cgetr(2); x[1]=evalexpo(bitprec); return x; }
+real_0_bit(long bitprec) { GEN x=cgetg(2, t_REAL); x[1]=evalexpo(bitprec); return x; }
 INLINE GEN
-real_0(long prec) { return real_0_bit(-prec2nbits(prec)); }
+real_0(long prec) { return real_0_bit(-prec); }
 INLINE GEN
 real_1_bit(long bit) { return real_1(nbits2prec(bit)); }
 INLINE GEN
 real_1(long prec) {
   GEN x = cgetr(prec);
-  long i;
+  long i, l = lg(x);
   x[1] = evalsigne(1) | _evalexpo(0);
-  x[2] = (long)HIGHBIT; for (i=3; i<prec; i++) x[i] = 0;
+  x[2] = (long)HIGHBIT; for (i=3; i<l; i++) x[i] = 0;
   return x;
 }
 INLINE GEN
 real_m1(long prec) {
   GEN x = cgetr(prec);
-  long i;
+  long i, l = lg(x);
   x[1] = evalsigne(-1) | _evalexpo(0);
-  x[2] = (long)HIGHBIT; for (i=3; i<prec; i++) x[i] = 0;
+  x[2] = (long)HIGHBIT; for (i=3; i<l; i++) x[i] = 0;
   return x;
 }
 
@@ -1765,7 +1789,7 @@ remis(GEN x, long y)
 {
   pari_sp av = avma;
   long r;
-  (void)divis_rem(x,y, &r); set_avma(av); return stoi(r);
+  (void)divis_rem(x,y, &r); return gc_stoi(av, r);
 }
 
 INLINE GEN
@@ -1796,20 +1820,20 @@ rdivss(long x, long y, long prec)
 INLINE void
 rdiviiz(GEN x, GEN y, GEN z)
 {
-  long prec = realprec(z), lx = lgefint(x), ly = lgefint(y);
+  long lz = lg(z), lx = lgefint(x), ly = lgefint(y);
   if (lx == 2) { affur(0, z); return; }
   if (ly == 3)
   {
     affir(x, z); if (signe(y) < 0) togglesign(z);
     affrr(divru(z, y[2]), z);
   }
-  else if (lx > prec + 1 || ly > prec + 1)
+  else if (lx > lz + 1 || ly > lz + 1)
   {
     affir(x,z); affrr(divri(z, y), z);
   }
   else
   {
-    long b = bit_accuracy(prec) + expi(y) - expi(x) + 1;
+    long b = lg2prec(lz) + expi(y) - expi(x) + 1;
     GEN q = divii(b > 0? shifti(x, b): x, y);
     affir(q, z); if (b > 0) shiftr_inplace(z, -b);
   }
@@ -2215,7 +2239,7 @@ affsr(long x, GEN y)
 
   if (!x)
   {
-    y[1] = evalexpo(-prec2nbits(ly));
+    y[1] = evalexpo(-bit_accuracy(ly));
     return;
   }
   if (x < 0) {
@@ -2237,7 +2261,7 @@ affur(ulong x, GEN y)
 
   if (!x)
   {
-    y[1] = evalexpo(-prec2nbits(ly));
+    y[1] = evalexpo(-bit_accuracy(ly));
     return;
   }
   sh = bfffo(x);
